﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.BusinessLogicContracts;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using BlacksmithWorkshopDataModels.Enums;
using Microsoft.Extensions.Logging;

namespace BlacksmithWorkShopBusinessLogic.BusinessLogics
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage orderStorage)
        {
            _logger = logger;
            _orderStorage = orderStorage;
        }
        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("Readlist. Id: {Id}.", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("Readlist. Count: {Count}", list.Count);
            return list;
        }
        public bool SetNewStatus(OrderBindingModel model, OrderStatus newstatus)
        {
            CheckModel(model, false);
            OrderViewModel? vm = _orderStorage.GetElement(new()
            {
                Id = model.Id
            });
            model.Status = vm?.Status ?? OrderStatus.Неизвестен;
            if ((int)model.Status == (int)newstatus - 1)
            {
                model.Status = newstatus;
                if (newstatus == OrderStatus.Готов)
                {
                    model.DateImplement = DateTime.Now;
                }
                if (_orderStorage.Update(model) != null)
                {
                    return true;
                }
            }
            _logger.LogWarning($"Changing order status of order {model.Id} to {newstatus} failed");
            return false;
        }
        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status == OrderStatus.Неизвестен)
            {
                model.Status = OrderStatus.Принят;
                if (_orderStorage.Insert(model) != null)
                {
                    return true;
                }
            }
            _logger.LogWarning("Create order operation failed");
            return false;
        }
        public bool TakeOrderInWork(OrderBindingModel model) => SetNewStatus(model, OrderStatus.Выполняется);
        public bool FinishOrder(OrderBindingModel model) => SetNewStatus(model, OrderStatus.Готов);
        public bool DeliveryOrder(OrderBindingModel model) => SetNewStatus(model, OrderStatus.Выдан);
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество изделий должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Стоимость заказа должна быть больше 0", nameof(model.Count));
            }
            _logger.LogInformation("Order. Count: {Count}. Sum: {Sum}. Id: {Id}", model.Count, model.Sum, model.Id);
            var element = _orderStorage.GetElement(new OrderSearchModel
            {
                Id = model.Id
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Изделие с таким ID уже есть");
            }
        }
    }
}